from numpy import *
def wgs2utm(Lat=None, Lon=None):
# -------------------------------------------------------------------------
# x,y,utmzone = wgs2utm(Lat,Lon)
#
# Description:
#    Convert WGS84 coordinates (Latitude, Longitude) into UTM coordinates
#    (northing, easting).
#
# Input:
#    Lat: WGS84 Latitude scalar or vector in decimal degrees  
#    Lon: WGS84 Longitude scalar or vector in decimal degrees  
#
# Output:
#    x: UTM easting in meters
#    y: UTM northing in meters
#    utmzone: UTM longitudinal zone
#
# Author notes:
#    I downloaded and tried deg2utm.m from Rafael Palacios but found
#    differences of up to 1m with my reference converters in southern
#    hemisphere so I wrote my own code based on "Map Projections - A
#    Working Manual" by J.P. Snyder (1987). Quick quality control performed
#    only by comparing with LINZ converter
#    (www.linz.govt.nz/apps/coordinateconversions/) and Chuck Taylor's 
#    (http://home.hiwaay.net/~taylorc/toolbox/geography/geoutm.html) on a 
#    few test points, so use results with caution. Equations not adapted
#    for a latitude of +/- 90deg.
#
# Example:
#    Lat=[48.866667, 34.05,   -36.85 ]
#    Lon=[2.333056,  -118.25, 174.783333]
#    x,y,utmzone = wgs2utm(Lat,Lon)
#
# Author: 
#   Alexandre Schimel
#   Coastal Marine Group - University of Waikato
#   Hamilton, New Zealand
#
# Version:
# April 2007
# Rewritten in Python by Orlando Camargo Rodriguez
# Faro, Ter 23 Fev 2021 20:53:58 WET 
#-------------------------------------------------------------------------

## coordinates in radians
    lat = Lat*pi/180.0
    lon = Lon*pi/180.0

## WGS84 parameters
    a = 6378137.0         #semi-major axis
    b = 6356752.314245    #semi-minor axis
    e = sqrt(1-(b/a)**2)  # eccentricity
    exe = e**2
## UTM parameters
    Lon0 = floor(Lon/6.0)*6+3 # reference longitude in degrees
    lon0 = Lon0*pi/180.0      # in radians
    k0 = 0.9996               # scale on central meridian

    FE = 500000.0             # false easting
#   FN = (Lat < 0)*10000000.0 # false northing - Matlab way...
    FN = where( Lat < 0, 1, 0 )*10000000.0 # false northing
## Equations parameters
    eps = exe/( 1.0 - exe )   # e prime square
# N: radius of curvature of the earth perpendicular to meridian plane
# Also, distance from point to polar axis
    N = a/sqrt( 1.0 - exe*sin(lat)**2) 
    T = tan(lat)**2
    C = eps*(cos(lat))**2
    A = (lon-lon0)*cos(lat)
    AxA = A**2
    TxT = T**2
# M: true distance along the central meridian from the equator to lat
    M = a*(  ( 1.0 - exe/4.0      -  3.0*exe**2/64.0 -  5.0*exe**3/256.0  ) * lat 
        - (      3.0*exe/8.0      +  3.0*exe**2/32.0 + 45.0*exe**3/1024.0 ) * sin(2.0*lat) 
        + (     15.0*exe**2/256.0 + 45.0*exe**3/1024.0 ) * sin(4.0*lat) 
        - (     35.0*exe**3/3072.0 )                     * sin(6.0*lat) )

## easting
    x = FE + k0*N*( A + ( 1.0 - T + C )*A**3/6.0 + ( 5.0 - 18.0*T + TxT + 72.0*C - 58.0*eps )*A**5/120.0 )
               
## northing
# M(lat0) = 0 so not used in following formula
    y = FN + k0*M + k0*N*tan(lat)*( AxA/2.0 + ( 5.0 - T + 9.0*C + 4.0*C**2 )* AxA**2/24.0 + ( 61.0 - 58.0*T + TxT + 600.0*C - 330.0*eps)*AxA**3/720.0 )

## UTM zone
    utmzone = ( floor(Lon0/6.0) + 31 ).astype(int)
    return x,y,utmzone
